@@ -29,6 +29,7 @@
 #include "grn_plugin.h"
 #include "grn_hash.h"
 #include "grn_ctx_impl.h"
+#include "grn_util.h"
 
 #ifdef WIN32
 # include <io.h>
@@ -1223,9 +1224,21 @@ grn_io_seg_map_(grn_ctx *ctx, grn_io *io, uint32_t segno, grn_io_mapinfo *info)
     fileinfo *fi = &io->fis[fno];
     if (!grn_fileinfo_opened(fi)) {
       char path[PATH_MAX];
+      grn_bool path_exist = GRN_TRUE;
       gen_pathname(io->path, path, fno);
+      path_exist = grn_path_exist(path);
       if (!grn_fileinfo_open(ctx, fi, path, O_RDWR|O_CREAT)) {
         DO_MAP(io, &info->fmo, fi, pos, segment_size, segno, info->map);
+        if (!info->map && !path_exist) {
+          if (grn_unlink(path) == 0) {
+            GRN_LOG(ctx, GRN_LOG_INFO,
+                    "[io][map][error] memory mapping is failed and then "
+                    "removed created map file: <%s>", path);
+          } else {
+            ERRNO_ERR("[io][map][error] memory mapping is failed and then "
+                      "failed to remove created map file: <%s>", path);
+          }
+        }
       }
     } else {
       DO_MAP(io, &info->fmo, fi, pos, segment_size, segno, info->map);
@@ -1829,6 +1842,10 @@ grn_fileinfo_open(grn_ctx *ctx, fileinfo *fi, const char *path, int flags)
 
   rc = grn_fileinfo_open_common(ctx, fi, path, flags);
   if (rc != GRN_SUCCESS) {
+    if (fi->fh) {
+      CloseHandle(fi->fh);
+      fi->fh = INVALID_HANDLE_VALUE;
+    }
     return rc;
   }
 
